#include "StdAfx.h"
#include "AqBusDecodeAnalysis.h"

//====================================================================
void AQcI64Array::SetSize(INT_PTR nNewSize, INT_PTR nGrowBy)
{  ARINT*  pUI64NewData;

	if(0 < nGrowBy)
	  m_nGrowBy = nGrowBy;

	
	if(nNewSize > 0 && nNewSize > m_iPtr)
	{ 
	  pUI64NewData = (ARINT*)new ARINT[nNewSize];
	  memset(pUI64NewData, 0, sizeof(ARINT)*nNewSize);
	  // Copy old data to new buffer
	  if(pUI64NewData)
	  {
	    if(m_pI64Data)
		{
          memcpy(pUI64NewData, m_pI64Data, sizeof(ARINT)*m_nMaxSize);
		  m_iPtr = m_nMaxSize;
		}		
	  }
	  else
	    return;
	  
	  if(m_pI64Data)	   
		 delete [] m_pI64Data;		
	  
	  m_pI64Data = pUI64NewData;
	  m_nMaxSize = nNewSize;
	}
}
AQcI64Array::AQcI64Array()
{  
   m_iPtr = 0; 
   m_pI64Data = NULL; 
   m_nMaxSize = 0; 
   m_nGrowBy = -1;
};
AQcI64Array::~AQcI64Array()
{
	if(m_pI64Data)
		delete [] m_pI64Data;
}
ARINT AQcI64Array::Add( ARINT nData )
{ 
	if( m_nMaxSize == GetSize())
	{
		if(m_nGrowBy == -1)
			SetSize(m_nMaxSize+1024);
		else
		    SetSize(m_nMaxSize+m_nGrowBy);
	}
	SetAt( m_iPtr, nData ); 
	m_iPtr++;
	return m_iPtr-1;
}

#ifdef _DEBUG
void    AQcI64Array::SetAt(ARINT nIndex, ARINT newElement) 
{
	if(nIndex < 0 || nIndex > m_nMaxSize)
		ASSERT(0);

	m_pI64Data[nIndex] = newElement; 
}
ARINT   AQcI64Array::GetAt(ARINT nIndex) 
{ 
	if(nIndex < 0 || nIndex > m_nMaxSize)
		ASSERT(0);
	return m_pI64Data[nIndex]; 
}
#endif
//====================================================================

cAqBusDecodeAnalysis::cAqBusDecodeAnalysis(void)
{
	m_hMapFile = NULL;
}

cAqBusDecodeAnalysis::~cAqBusDecodeAnalysis(void)
{
	if( m_hMapFile )
		CloseHandle(m_hMapFile);
}

bool cAqBusDecodeAnalysis::Analysis(LPDECODECFG lpDecodeCfg)
{
	cAqBusDecodeSetting* pAqBusDecodeSetting;

	m_lpDecodeCfg = lpDecodeCfg;

	// if no signal, return false
	if( wfGetStatus() & WF_NO_SIGNAL )
		return false;

	pAqBusDecodeSetting = new cAqBusDecodeSetting(lpDecodeCfg);

	if( !m_iaData.IsEmpty() )
	{
		m_iaData.RemoveAll();
		m_iaBoundary.RemoveAll();
	}

	// XҦ}쪺]w
	m_iChClk = pAqBusDecodeSetting->GetChClk();
	m_iChData = pAqBusDecodeSetting->GetChData();

	// setting begin & end
	if( pAqBusDecodeSetting->GetBegin() )
		m_iBegin = m_lpDecodeCfg->i64LaCsr[pAqBusDecodeSetting->GetBegin()-1];
	else
		m_iBegin = 0;

	if( pAqBusDecodeSetting->GetEnd() )
		m_iEnd = m_lpDecodeCfg->i64LaCsr[pAqBusDecodeSetting->GetEnd()-1];
	else
		m_iEnd = m_lpDecodeCfg->i64LaSamples;

	// initialize and prepare to analysis
	wfInit(m_iBegin, m_iEnd);

	// Start Analysis
	AnalysisLoop();

	// analysis complete
	wfEnd();

	delete pAqBusDecodeSetting;

	// if there is no result, or the size is not match, return false
	if( m_iaBoundary.IsEmpty() )
		return false;

	if( m_iaData.GetSize() != m_iaBoundary.GetSize() )
		return false;
	
	return true;
}

bool cAqBusDecodeAnalysis::AnalysisLoop()
{
	int iPreChClk, iPreChData;
	int iChClk, iChData;
	int iEdgeClk, iEdgeData;
	int iPreEdgeClk, iPreEdgeData;

	if( !wfGetData(&m_i64CurrIdx, m_pWaveBuf) )
		return false;

	utlInitProgress(0, 500, wfGetEndIdx()-wfGetBeginIdx()+1);		// R 50 %(0~50)

	// ŪU}
	iPreChClk = GET_TR_1CH_DATA(m_pWaveBuf, m_iChClk);
	iPreChData = GET_TR_1CH_DATA(m_pWaveBuf, m_iChData);
	iPreEdgeClk = 0;
	iPreEdgeData = 0;
	m_i64Boundary = -1;

	while( wfGetNext(&m_i64CurrIdx, m_pWaveBuf) )
	{
		if( wfGetStatus() & WF_FORCE_STOP )
			break;

		utlSetProgress(m_i64CurrIdx);		// sprogress

		// ŪU}
		iChClk = GET_TR_1CH_DATA(m_pWaveBuf, m_iChClk);
		iChData = GET_TR_1CH_DATA(m_pWaveBuf, m_iChData);
		iEdgeClk = iChClk - iPreChClk;
		iEdgeData = iChData - iPreChData;

		// Analysis here
		// TODO:
#if 1
		// Sample
		m_iaData.Add(iChClk+iChData);
		m_iaBoundary.Add(m_i64CurrIdx);
#endif

		iPreChClk = iChClk;
		iPreChData = iChData;
		iPreEdgeClk = iEdgeClk;
		iPreEdgeData = iEdgeData;
		//m_i64Boundary = m_i64CurrIdx;
	}
	return false;
}

int cAqBusDecodeAnalysis::ConvertMsg(LPDECODECFG lpDecodeCfg, int *piData, char* pszBuf, int iBufSize, int *piColorIdx)
{
	int iIdx, iVal, iDataCnt, iData;
	char szVal[20];
	int iColorVar;
	CColorPicker ClrInfo;
	cAqBusDecodeSetting* pAqBusDecodeSetting;
	
	pAqBusDecodeSetting = new cAqBusDecodeSetting(lpDecodeCfg);

	// TODO:
#if 1
	// Sample
	iDataCnt = piData[0];
	iData = piData[1];
	iIdx = HIWORD(iData);
	iVal = LOWORD(iData);

	utlValueToStr_Report(iVal, lpDecodeCfg->iLaValType, 8, szVal, sizeof(szVal));

	sprintf_s(pszBuf, iBufSize, "%s", szVal);
#endif

	// Set Color
	if( iIdx < 16 )
	{
		iColorVar = pAqBusDecodeSetting->GetColor(iIdx);
		*piColorIdx = iColorVar;
	}
	else
		*piColorIdx = -1;	// ܤХC

	delete pAqBusDecodeSetting;

	return 0;
}

void cAqBusDecodeAnalysis::BuildCSV(LPDECODECFG lpDecodeCfg, I64 *pi64RetBoundary, UINT *pnRetData, int iRetDataSize)
{
	m_lpDecodeCfg = lpDecodeCfg;

	// Close old map file
	if(m_hMapFile)
	   CloseHandle( m_hMapFile );
	
	if( rptBegin(iRetDataSize, true) )
	{
		BuildCSVGo(pi64RetBoundary, pnRetData, iRetDataSize);
	}

	m_hMapFile = rptClose();
}

char* iRptFieldStrLst[] = 
{
	"Field 1",
	"Field 2",
	"Field 3",
	0
};

enum
{
	RPT_COLUMN_1 = 0,
	RPT_COLUMN_2,
	RPT_COLUMN_3,
};

void cAqBusDecodeAnalysis::BuildCSVGo(I64 *pi64RetBoundary, UINT *pnRetData, int iRetDataSize)
{
	int iSize = (int)m_iaBoundary.GetSize();
	int iColorVar;
	int iField, iData;
	CColorPicker ClrInfo;
	TRIDX i64CurrBoundary, i64LastBoundary;
	cAqBusDecodeSetting* pAqBusDecodeSetting;
	char szTmp[50], szDataColumn[50];
	
	// set title
	for( int i=0 ; iRptFieldStrLst[i] ; i++ )
		rptSetHeaderColumnStr(i, iRptFieldStrLst[i]);

	pAqBusDecodeSetting = new cAqBusDecodeSetting(m_lpDecodeCfg);

	// set color table
	for( int i = 0 ; i < COLOR_COUNT ; i++ )
	{
		iColorVar = pAqBusDecodeSetting->GetColor(i);
		rptSetColorTable(i, ClrInfo.GetInvColor(iColorVar), ClrInfo.GetColor(iColorVar));
	}
	rptSetColorTable(COLOR_COUNT, RGB(0, 0, 0), RGB(255, 255, 255));		// թ¦r
	rptSetColorTable(COLOR_COUNT+1, RGB(255, 255, 255), RGB(255, 0, 0));	// Errorܬզr
	
	delete pAqBusDecodeSetting;

	rptBuildHeader();

	// set the column color index
	for( int i = 0 ; iRptFieldStrLst[i] ; i++ )
		rptSetColumnColorIdx(i, COLOR_COUNT);

	// stop if no any data
	if( iSize == 0 )
		return;

	utlInitProgress(750, 1000, iSize);	// إreport25 %	(75~100)

	i64LastBoundary = -1;
	for(int i = 0 ; i < iSize ; i++)
	{
		if( wfGetStatus() & WF_FORCE_STOP )								// j			
		  break;

		utlSetProgress(i);		// 75% ~ 100%

		// build report
		// TODO:
#if 1
		// sample
		iField = HIWORD(pnRetData[i]);
		iData = LOWORD(pnRetData[i]);
		i64CurrBoundary = pi64RetBoundary[i];

		if( i64LastBoundary != -1 )
		{
			rptSetColumnData(RPT_COLUMN_2, szDataColumn);
			rptBuildColumnData(); // write last data
		}

		rptSetColumnBoundary(i64CurrBoundary);
		utlValueToStr_Report(iData, m_lpDecodeCfg->iLaValType, 8, szTmp, sizeof(szTmp));
		rptSetColumnData(RPT_COLUMN_1, szTmp);
#endif

		i64LastBoundary = i64CurrBoundary;
	}

	rptSetColumnData(RPT_COLUMN_2, szDataColumn);	// Sample
}
