#ifndef _PUBLIC_LIB_H_
#define _PUBLIC_LIB_H_

#define DECODE_CSR_CNT		6
#define DECODE_CH_CNT		256
#define DECODE_PARAM_CNT	3

// DECODECFG.wFlag
#define WFLAG_USED 0x01

// Get Waveform
// wfGetStatus()
#define WF_NO_SIGNAL	0x01
#define WF_FORCE_STOP	0x02

#define GET_TR_1CH_DATA(Buf, Ch) ((Buf[Ch>>3]>>(Ch&7)) & 1)

// Error Code
#define LA_ERR_NONE					0		// None Error
#define LA_ERR_CFG_NOT_SET			1		// Config data is not set
#define LA_ERR_CLASS_CREATE_FAIL	2		// Internal Class create fail
#define LA_ERR_RPT_COL_IDX			3		// Column Index is out of range
#define LA_ERR_RPT_STRING			4		// Input String point is NULL
#define LA_ERR_RPT_COLOR_IDX		5		// Color Index is out of range
#define LA_ERR_RPT_BUILD_HEAD		6		// Build Header error
#define LA_ERR_NULL_HWND			7		// hWnd is NULL
#define LA_ERR_SEND_MSG_FAIL		8		// SendMessage fail
#define LA_ERR_BUF_NULL				9		// Waveform buffer point is NULL

// iLaValType - LA Value Type
#define LA_VALUE_TYPE_HEX			0
#define LA_VALUE_TYPE_DEC			1
#define LA_VALUE_TYPE_OCT			2
#define LA_VALUE_TYPE_BIN			3
#define LA_VALUE_TYPE_ASC			4
#define LA_VALUE_TYPE_ANALOG		6

#ifndef UI64
#define UI64	unsigned __int64
#define I64		__int64
#endif

typedef __int64 TRIDX;
typedef TRIDX ARINT;

#define TR_BUF_SIZE 16

typedef struct _DECODECFG
{
	UINT			cbSize;						// size of this structure
	// LAǻDecodeT
	int				iLaMaxCh;					// [I] Number of LA Available Channel.
	I64				i64LaSamples;				// [I] This is total samples for decode.
	I64				i64LaCsr[DECODE_CSR_CNT];	// [I] The cursor position by samples, 0 is T Cursor, 1 is A cursor, 2 is B Cursor etc.
	int				iLaValType;					// [I] The Number of display type (BIN,DEC,OCT,HEX,ASC....)
	int				iNumOfUsedChannel;			// [O] Number of Used Channel.
	HWND			hWnd;						// [I] The Window handle of the LA Viewer.
	int				iLaSettings;				// [i] The Settings information from LA Viewer

	//UDecode^ǵLAT
	BOOL			fRetCode;					// [O] use for _SettingsDialog() & _DefaultSetting(), return true is OK, false is CANCEL
	I64				i64SampleRate;				// [I] Sample Rate

	// Bus decode settings
	WORD			wFlag;						// [I/O] The Bus decode settings parameter
	int				iCh[DECODE_CH_CNT];			// [I/O] Channel ID
	int				iPara[DECODE_PARAM_CNT];	// [I/O] The Bus decode settings parameter
} DECODECFG, FAR *LPDECODECFG;


#ifndef  CColorPicker
/////////////////////////////////////////////////////////////////////////////
// CColorPicker window

class CColorPicker : public CComboBox
{
// Construction
public:
	CColorPicker();
	BOOL Initialize();

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CColorPicker)
	public:
	virtual void DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CColorPicker();
	COLORREF GetColor(int iIdx);
	COLORREF GetInvColor(int iIdx);
	

	// Generated message map functions
protected:
	//{{AFX_MSG(CColorPicker)
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};
#endif


/////////////////////////////////////////////////////////////////////////////
// Get Waveform
bool wfInit(TRIDX iBegin, TRIDX iEnd);
int wfGetStatus();
TRIDX wfGetBeginIdx();
TRIDX wfGetEndIdx();
bool wfGetData(TRIDX *index, LPBYTE lpb);
bool wfGetNext(TRIDX *index, LPBYTE lpb);
int wfCalcBitRate(int iCh0, __int64 i64MinWidth_Hz);
void wfSetMaskData(LPBYTE lpb);
void wfSetMaskData1Ch(int iCh);
void wfSetMaskDataClr();
bool wfEnd();

/////////////////////////////////////////////////////////////////////////////
// Build Report
#ifndef rptBegin
bool rptBegin(int iBoundarySize, bool fFileMode);
#endif
HANDLE rptClose();
// Set the header info
bool rptSetHeaderColumnStr(int iColumnIdx, char* szHeadStr);
bool rptSetColorTable(int iColorIdx, COLORREF ColorFG, COLORREF ColorBG);
bool rptBuildHeader();
void rptSetColumnBoundary(TRIDX iBoundary);
bool rptSetColumnColorIdx(int iColumnIdx, int iColorIdx);
bool rptSetColumnData(int iColumnIdx, char* szColumnStr);
bool rptSetColumnStrLength(int iColumnIdx, int iLength);
void rptClrColumnData();
void rptBuildColumnData();
void rptGetColorStr(int iColor, char* szOutputStr);

/////////////////////////////////////////////////////////////////////////////
// use for Build Report/Convert Message
DWORD utlGrayCode( DWORD dwValue );
UI64 utlGrayCode64(UI64 i64Value );
INT utlValueToStr( DWORD dwValue, int iType, int iDigits, char *szStr, int iBufSize);
INT utlValue64ToStr( DWORD dwValueLo, DWORD dwValueHi,int iType, int iDigits, char *szStr, int iBufSize);
INT utlValueToStr_Report( DWORD dwValue, int iType, int iDigits, char *szStr, int iBufSize);
INT utlValue64ToStr_Report( DWORD dwValueLo, DWORD dwValueHi,int iType, int iDigits, char *szStr, int iBufSize);
int utlBuildAsc(char* szOutputBuf, char* szSrcBuf, int iDataLen);
char* utlStrUpr(char * szString);

/////////////////////////////////////////////////////////////////////////////
// use for Progress bar
void utlInitProgress(int iMin, int iMax, I64 i64Step);
void utlSetProgress(I64 i64Current);

/////////////////////////////////////////////////////////////////////////////
// Get WorkDir
bool utlGetWorkDir(HWND hWnd, char *szFilePathName);

/////////////////////////////////////////////////////////////////////////////
// Get Error Code
int utlGetErrorCode();

/////////////////////////////////////////////////////////////////////////////
// Get Version
int utlGetVersion();

#define MAKEUI64(a,b)  ((UI64)(((DWORD)(((UI64)(a)) & 0xffffffff)) | ((UI64)((DWORD)(((UI64)(b)) & 0xffffffff))) << 32)) 

/////////////////////////////////////////////////////////////////////////////
// functionBus DecodeTLAD{ϥΡA^Ǩϥγ̤jqDơC
// parameter :
//	int* piMaxCh		: [O] qDs̤jh. (ҦpDSOuBz6ӳqD), J-1NHLA
//	char *szName		: [O] W
//	int iNameBufSize	: [I] szNamesize
//	int** piRptField	: [O] Reporteת, ̫@ӻݬs. ҦpProtocolT: 100, 150, 120, 0
// return :
//	^Ǩϥγ̤jqD
int _GetDecodeInfo(int* piMaxCh, char *szName, int iNameBufSize, int** piRptField);

// function \ରi Protocol Ru@,
// ̫إߤRƤ Boundary/Data arrayCarrayƻݫOsDLL
// Parameter :
//	LPDECODECFG lpDecodeCfg	: [I] decode config
//	I64 **pi64RetBoundary	: [O] Boundary array
//	UINT **piRetData		: [O] Data array
// return :
//	size of array
int _Analysis(LPDECODECFG lpDecodeCfg, I64 **pi64RetBoundary, UINT **piRetData);

// function \ର Protocol R, ̷Boundary/Data arrayReport
// ̫ᲣͤRƤ Report DataC
// Parameter :
//	LPDECODECFG lpDecodeCfg	: [I] decode config
//	I64 *pi64RetBoundary	: [I] Boundary array
//	UINT *piRetData			: [I] Data array
//	int iRetDataSize		: [I] Size of array
BOOL _BuildReport(LPDECODECFG lpDecodeCfg, I64 *pi64RetBoundary, UINT *piRetData, int iRetDataSize);

// function \ണܩiΰϪr,
// LA Viewer |eeWRƤ Data ǤJC
// Parameter :
//	LPDECODECFG lpDecodeCfg	: [I] decode config
//	int *piData				: [I] Data ValueơA1stslengthC
//	char *szBuf				: [O] xsܩiΰϤrƪwİϦ}C
//	int iBufSize			: [I] szBufҰѦҤwİϤjpC
//	int *iColorIdx			: [O] Ǧ^ХܪCN(0-15, -1NХC)
BOOL _DataConv(LPDECODECFG lpDecodeCfg, int *piData, char *szBuf, int iBufSize, int *piColorIdx);

// function \O^ǳqDW, HKLA Viewer ܩeWC
// Parameter :
//	LPDECODECFG lpDecodeCfg	: [I] decode config
//	int iChIdx				: [I] qDsC
//	char *szBuf				: [O] xsqDW٤wİϦ}C
//	int iBufSize			: [I] szBufҰѦҤwİϤjpC
BOOL _ChannelName(LPDECODECFG lpDecodeCfg, int iChIdx, char *szBuf, int iBufSize);

// function \ରֳt Settings, |} Settings DialogC
// Parameter :
//	LPDECODECFG lpDecodeCfg	: [I/O] decode config
void _DefaultSetting(LPDECODECFG lpDecodeCfg);

// function \O} Settings e,
// J Bus decode ݭnϥΪ̨Mw	Ѽ. Bus decode bq} Settings dialogHK󱵦]wѼơC
// Parameter :
//	LPDECODECFG lpDecodeCfg	: [I/O] decode config
void _SettingsDialog(LPDECODECFG lpDecodeCfg);

// function \O Protocol ^ display name
// Le _GetDecodeInfo() Ҧ^ǪW٤PO
// Display name Hɥiק,  Bus decode name @wF
// NiHק. _h¦sɷ|䤣W, y}ɥ
BOOL _GetDisplayName(char* pNameBuf, int iBufSize);


// tX QuickSet, ݶJ LA  default Ѽ.
// Lצp, dwLAChNum @wnqD.
// Parameter :
//		DWORD* pdwLAChNum: [I/O] ^Ǧ decode һ LA qD
//		DWORD* pdwLAThres: [I/O] ^Ǧ decode w] LA Threshold, O mV
//		DWORD* pi64LASR  : [I/O] ^Ǧ decode w] LA Sample rate, O Hz. Y 0 
//				   		          Υثḛ sample rate
BOOL _GetLAQuickSetInfo(DWORD* pdwLAChNum, DWORD* pdwLAThres, INT64* pi64LASR);

// tX QuickSet, ݶJ DSO  default Ѽư򥻸.
// Lצp, pdwDSOChNum @wnqD.
// Parameter :
//		DWORD* pdwLAChNum: [I/O]      ^Ǧ decode һ DSO qD
//				                      YS, Ц^ 0.  return false
//		DWORD* pdwDSOFreqInKHz: [I/O] ^Ǧ decode w] DSO Sample rate, O KHz. 
//				                      i 0
BOOL _GetDSOQuickSetInfo(DWORD* pdwDSOChNum, DWORD* pdwDSOFreqInKHz);

// tX QuickSet, Ywg^ DSO Channel number , NJ
// n۰ʷsWqDW
// Parameter :
//		int iIdx: [I]			qD,  0 }l
//		char* pChNameBuf: [I/O] ^ǳqDW string buffer
//		int iBufSize: [I] ^ǳqDW string buffer size
BOOL _GetDSOQuickSetInfoName(int iIdx, char* pChNameBuf, int iBufSize);

#endif	//_PUBLIC_LIB_H_